// #autoload
// #name = MojoMark
// #version = 1.0b
// #date = January 21, 2001
// #author = Halide
// #warrior =  Halide
// #email = halide@carolina.rr.com
// #web = http://halides.tribes-universe.com/
// #description = FPS benchmark updated September 5, 2003 for classic
// #category = Mojoisasexybeast
// #status = working
// #credit = Ohm|-A3- for the classic update
// #credit = Mojoman- of iM fame for the FPS challenging demo


if(!isObject(HalMark))
{
	new ScriptObject(HalMark)
	{
		class = HalMark;
		lvlFPS[1] = 0;	// fps for each level
		lvlFPS[2] = 25;
		lvlFPS[3] = 40;
		colFPS[1] = "dd0000"; // red
		colFPS[2] = "dddd00"; // yellow
		colFPS[3] = "00dd00"; // green

		updateRate = 100;
		timeScale = 1;
	};
}

// START package HalMark
package HalMark
{

	function HalMark::start(%this)
	{
		%this.active = true;
		%this.startLoadTime = getRealTime();

		%file = "recordings/MojoMark_demo1"; // no extension

		Canvas.pushDialog(DemoLoadProgressDlg);

		if(playDemo(%file))
		{
			%this.totalLoadTime = (getRealTime() - %this.startLoadTime) / 1000;

			// do not allow new sources to have a force feedback effect
			alxEnableForceFeedback(false);

			resetGameState();
			canvas.setContent(panoramaGui);
			canvas.cursorOff();
			loadDemoSettings();

			// setup the global action map
			GlobalActionMap.bindCmd(keyboard, "escape", "stopDemoPlayback();", "");
			GlobalActionMap.bindCmd(keyboard, "space", "doScreenShot();", "");
			$globalActionMapOnly = true;

			$timeScale = %this.timeScale;
			%this.update();
		}
		else
			MessageBoxOK("Playback Failed", "Demo playback failed for MojoMark!");

		Canvas.popDialog(DemoLoadProgressDlg);
	}

	function demoPlaybackComplete()
	{
		parent::demoPlaybackComplete();

		if(HalMark.active)
		{
			canvas.popDialog(RecordingsDlg);

			HalMark.done();
		}
	}

	function HalMark::done(%this)
	{
		%this.score = mFloor(%this.totalFPS / 25 - %this.totalLoadTime * 2);
		MessageBoxOK("MojoMark Results", "Score: <spush><color:ffffff>" @ %this.score @ "<spop>\n\nLoad Time: <spush><color:ffffff>" @ mFormatFloat(%this.totalLoadTime, "%.1f") @ "s<spop>\nAverage FPS: <spush>" @ %this.getColFPS(%this.avgFPS) @ mFormatFloat(%this.avgFPS, "%.1f") @ "<spop>");
		%this.reset();
	}

	function HalMark::update(%this, %val)
	{
		if(%this.active)
			%this.schedule(%this.updateRate, update, true);
		else
			return;

		%this.curFPS = $fps::real;
		%this.avgFPS = (%this.totalFPS += %this.curFPS) / %this.totalPolls++;

		HalMarkCurFPS.setText("<just:center><font:sui generis:20>" @ %this.getColFPS(%this.curFPS) @ mFormatFloat(%this.curFPS, "%.1f"));
		HalMarkAvgFPS.setText("<just:center><font:sui generis:20>" @ %this.getColFPS(%this.avgFPS) @ mFormatFloat(%this.avgFPS, "%.1f"));
		HalMarkTime.setText("<just:center><font:sui generis:20>" @ mFormatFloat(%this.curTime += (%this.updateRate / 1000), "%.1f") @ "s");
	}

	function HalMark::reset(%this)
	{
		%this.active = false;
		%this.totalPolls = 0;
		%this.totalFPS = 0;
		%this.curFPS = 0;
		%this.avgFPS = 0;
		%this.curTime = 0;
	}
	
	function HalMark::getColFPS(%this, %val)
	{
		for(%i = 1; %i <= 3; %i++)
			if(%val <= %this.lvlFPS[%i])
				break;
		
		return "<color:" @ %this.colFPS[%i-1] @ ">";
	}

	function HalMark::setup(%this)
	{
		if(%this.setup)
			return;
		
		new GuiControl(HalMarkHud) {
			profile = "GuiDefaultProfile";
			horizSizing = "right";
			vertSizing = "bottom";
			position = "0 0";
			extent = "640 480";
			minExtent = "8 8";
			visible = "1";
			hideCursor = "0";
			bypassHideCursor = "0";
			helpTag = "0";

			new GuiBitmapCtrl(HalMarkBG) {
				profile = "GuiDefaultProfile";
				horizSizing = "right";
				vertSizing = "bottom";
				position = "0 0";
				extent = "280 50";
				minExtent = "8 8";
				visible = "1";
				hideCursor = "0";
				bypassHideCursor = "0";
				helpTag = "0";
				bitmap = "gui/MojoMarkBevel.png";
				wrap = "0";
			};
			new GuiMLTextCtrl(HalMarkCurFPS) {
				profile = "GuiDefaultProfile";
				horizSizing = "right";
				vertSizing = "bottom";
				position = "6 20";
				extent = "77 20";
				minExtent = "8 8";
				visible = "1";
				hideCursor = "0";
				bypassHideCursor = "0";
				helpTag = "0";
				lineSpacing = "2";
				allowColorChars = "0";
				maxChars = "-1";
				deniedSound = "InputDeniedSound";
			};
			new GuiMLTextCtrl(HalMarkAvgFPS) {
				profile = "GuiDefaultProfile";
				horizSizing = "right";
				vertSizing = "bottom";
				position = "89 20";
				extent = "77 20";
				minExtent = "8 8";
				visible = "1";
				hideCursor = "0";
				bypassHideCursor = "0";
				helpTag = "0";
				lineSpacing = "2";
				allowColorChars = "0";
				maxChars = "-1";
				deniedSound = "InputDeniedSound";
			};
			new GuiMLTextCtrl(HalMarkTime) {
				profile = "GuiDefaultProfile";
				horizSizing = "right";
				vertSizing = "bottom";
				position = "169 20";
				extent = "107 14";
				minExtent = "8 8";
				visible = "1";
				hideCursor = "0";
				bypassHideCursor = "0";
				helpTag = "0";
				lineSpacing = "2";
				allowColorChars = "0";
				maxChars = "-1";
				deniedSound = "InputDeniedSound";
			};
		};

		panoramaGui.add(HalMarkHud);

		%this.setup = true;
	}
	
	function DispatchLaunchMode()
	{
		parent::DispatchLaunchMode();

		HalMark.setup();

		LaunchToolbarMenu.insertItemAt(LaunchToolbarMenu.findItem(72, "SCRIPTS"), 76, "Run MojoMark", "HalMark.start();");
	}

// END package HalMark
};
activatePackage(HalMark);